﻿using Hotline.Application.Knowledge;
using Hotline.KnowledgeBase;
using Hotline.Permissions;
using Hotline.Share.Dtos;
using Hotline.Share.Dtos.Knowledge;
using Hotline.Share.Enums.KnowledgeBase;
using MapsterMapper;
using Microsoft.AspNetCore.Mvc;
using XF.Domain.Authentications;
using XF.Domain.Exceptions;

namespace Hotline.Api.Controllers
{
    /// <summary>
    /// 
    /// </summary>
    public class KnowledgeApplyController : BaseController
    {
        private readonly IKnowledgeRepository _knowledgeRepository;
        private readonly IKnowledgeApplyRepository _knowledgeApplyRepository;
        private readonly ISessionContext _sessionContext;
        private readonly IMapper _mapper;
        private readonly IKnowApplication _knowApplication;

        /// <summary>
        /// 
        /// </summary>
        /// <param name="knowledgeRepository"></param>
        /// <param name="knowledgeApplyRepository"></param>
        /// <param name="sessionContext"></param>
        /// <param name="mapper"></param>
        /// <param name="knowApplication"></param>
        public KnowledgeApplyController(IKnowledgeRepository knowledgeRepository, IKnowledgeApplyRepository knowledgeApplyRepository, ISessionContext sessionContext
            , IMapper mapper, IKnowApplication knowApplication)
        {
            _knowledgeRepository = knowledgeRepository;
            _knowledgeApplyRepository = knowledgeApplyRepository;
            _sessionContext = sessionContext;
            _mapper = mapper;
            _knowApplication = knowApplication;
        }

        /// <summary>
        /// 知识申请-发起申请
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [Permission(EPermission.KnowledgeApplyAdd)]
        [HttpPost("apply-add")]
        public async Task<string> ApplyAdd([FromBody] KnowledgeApplyDto dto)
        {
            var ka = _mapper.Map<KnowledgeApply>(dto);
            ka.Status = EKnowledgeApplyStatus.Handling;
            return await _knowledgeApplyRepository.AddAsync(ka, HttpContext.RequestAborted);
        }

        /// <summary>
        /// 知识申请-修改申请
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [Permission(EPermission.KnowledgeUpdateAdd)]
        [HttpPut("apply-update")]
        public async Task UpdateAdd([FromBody] UpdateKnowledgeApplyDto dto)
        {
            var ka = _mapper.Map<KnowledgeApply>(dto);
            await _knowledgeApplyRepository.UpdateAsync(ka, HttpContext.RequestAborted);
        }

        /// <summary>
        ///知识申请-信息办理
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        [Permission(EPermission.KnowApplyHandle)]
        [HttpPut("apply-handle")]
        public async Task ApplyHandle(string Id)
        {
            //查询申请信息
            var handle = await _knowledgeApplyRepository.GetAsync(Id, HttpContext.RequestAborted);

            if (handle is null)
                throw UserFriendlyException.SameMessage("办理失败");
            handle.Status = EKnowledgeApplyStatus.Succeed;
            //修改办理信息
            await _knowledgeApplyRepository.UpdateAsync(handle, HttpContext.RequestAborted);
        }

        /// <summary>
        ///知识申请- 撤销操作
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        [Permission(EPermission.KnowledgeApplyRevoke)]
        [HttpPut("apply-handle/{Id}")]
        public async Task ApplyRevoke(string Id)
        {
            var handle = await _knowledgeApplyRepository.GetAsync(Id, HttpContext.RequestAborted);
            if (handle is null)
                throw UserFriendlyException.SameMessage("申请信息撤销失败");
            handle.Status = EKnowledgeApplyStatus.Revert;
            await _knowledgeApplyRepository.UpdateAsync(handle, HttpContext.RequestAborted);
        }

        /// <summary>
        /// 知识申请-退回操作
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        [Permission(EPermission.KnowledgeApplyReturn)]
        [HttpPut("return")]
        public async Task ApplyReturn(string Id)
        {
            //查询申请信息
            var handle = await _knowledgeApplyRepository.GetAsync(Id, HttpContext.RequestAborted);
            if (handle is null)
                throw UserFriendlyException.SameMessage("退回失败");
            handle.Status = EKnowledgeApplyStatus.Failed;
            await _knowledgeApplyRepository.UpdateAsync(handle, HttpContext.RequestAborted);
        }

        /// <summary>
        /// 删除申请
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        [Permission(EPermission.KnowledgeApplyDelete)]
        [HttpDelete("delete")]
        public async Task ApplyDelete(string Id)
        {
            var handle = await _knowledgeApplyRepository.GetAsync(Id, HttpContext.RequestAborted);
            if (handle is null)
                throw UserFriendlyException.SameMessage("申请不存在");
            await _knowledgeApplyRepository.RemoveAsync(handle, false, HttpContext.RequestAborted);
        }

        /// <summary>
        /// 知识申请-查询详情
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        [HttpGet("info/{Id}")]
        public async Task<DataKnowledgeApplyDto> ApplyInfo(string Id)
        {
            var handle = await _knowledgeApplyRepository.GetAsync(Id, HttpContext.RequestAborted);
            if (handle is null)
                throw UserFriendlyException.SameMessage("申请信息查询失败");
            DataKnowledgeApplyDto dataKnowledgeApplyDto = new();
            _mapper.Map(handle, dataKnowledgeApplyDto);

            var knowledge = await _knowledgeRepository.GetAsync(p => p.Applies.Contains(Id), HttpContext.RequestAborted);
            if (knowledge != null)
            {
                dataKnowledgeApplyDto.KBID = knowledge.Id;
                dataKnowledgeApplyDto.KBTitle = knowledge.Title;
            }

            return dataKnowledgeApplyDto;
        }

        #region 我的知识申请
        /// <summary>
        /// 我的知识申请--全部
        /// </summary>
        /// <returns></returns>
        [Permission(EPermission.KnowledgeAllApply)]
        [HttpGet()]
        public async Task<PagedDto<KnowledgeApply>> GetApplyList([FromQuery] KnowledgeApplyPagedDto dto)
        {
            return await _knowApplication.GetApplyList(dto, HttpContext.RequestAborted);
        }

        #endregion

        #region 知识申请处理查询
        /// <summary>
        /// 申请处理查询
        /// </summary>
        /// <returns></returns>
        [HttpGet("handlelist")]
        public async Task<PagedDto<KnowledgeApplyPageDto>> AllHandle([FromQuery] KnowledgeHandlePagedDto dto)
        {
            return await _knowApplication.GetApplyHandleList(dto, HttpContext.RequestAborted);
        }

        #endregion
    }
}
